/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS Demuxkey API
 */
#ifndef _NODEOS_DEMUXKEY_H_
#define _NODEOS_DEMUXKEY_H_

#include <sys/types.h>
#include <nodeos/flow.h>

typedef struct {
	int stuff;
} *an_demuxkey_t;

/* XXX compat */
#ifdef NODEOS_SPEC_COMPAT
#define an_Demuxkey an_demuxkey_t
#define an_demuxkeyCreate an_demuxkey_create
#define an_demuxkeyDestroy an_demuxkey_destroy
#define an_demuxkeyMatch an_demuxkey_match
#endif

/*
 * Create an empty demux key (packet filter) object.
 * Returns a pointer to the new object or zero on error.
 */
an_demuxkey_t an_demuxkey_create(an_flow_t flow);

/*
 * Destroy a demux key.
 * Frees any resources allocated by an_demuxkey_match operations.
 * XXX make sure demux key is not "in use".
 */
void an_demuxkey_destroy(an_demuxkey_t key);

/*
 * Adds an <offset, length, matchdata> triple to the indicated demux.
 * The match data buffer is currently copied and hence may be overwritten
 * upon return from this call.  The mask param is currently ignored.
 */
int an_demuxkey_match(an_demuxkey_t key, int off, int len, void *data,
		      void *mask);

#endif /* _NODEOS_DEMUXKEY_H_ */
