/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Implementation specific types and constants for the Utah OSKit-based NodeOS.
 * Note that these are not machine dependencies, this file is used for both
 * the x86 and ARM versions.
 */
#ifndef _NODEOS_IMPL_H_
#define _NODEOS_IMPL_H_

#include <sys/types.h>
#include <sys/time.h>

/*
 * XXX hack global constants that should be specified in other ways.
 */
#define NODEOS_PAGESIZE		4096	/* memory page size */
#define NODEOS_CHAN_BUFSIZE	2048	/* channel buffer size */


typedef int an_error_t;
#ifdef OSKIT
typedef struct oskit_timespec an_timespec_t;
#else
typedef struct timespec an_timespec_t;
#endif

/*
 * Called when a child flow is being terminated.
 */
typedef void (*an_flowterm_func_t)(void *);

/*
 * Initialization information for the flow.
 * For us, it is just the routine to call to initialize the flow.
 * Might also include an argument vector or other useful stuff.
 */
typedef void (*an_flowinit_t)(void *);

/*
 * Called when a mempool is over limit
 */
typedef void (*an_mempoolfull_func_t)(void *, int, int);

/*
 * Not yet
 */
typedef void *an_sched_t;

/*
 * Thread interrupt handling
 * Includes API call to set the handler and signature for handler.
 */
typedef void (*an_thread_intr_t)(int, void *);

/* types passed to the notify function */
#define AN_THREAD_TRAP	1
#define AN_THREAD_INTR	2

void an_thread_sethandler(an_thread_intr_t handler);

/* XXX compat */
#ifdef NODEOS_SPEC_COMPAT
#define an_Error an_error_t
#define an_Timespec an_timespec_t
#define an_Schedule an_sched_t
#define an_NotifyFunc an_flowterm_func_t
#define an_FlowNotifyFunc an_flowterm_func_t
#define an_FlowStartSpec an_flowinit_t
#define an_MempoolNotifyFunc an_mempoolfull_func_t
#define an_ThreadNotifyFunc an_thread_intr_t
#define an_threadSetHandler an_thread_sethandler
#endif

#endif /* _NODEOS_IMPL_H_ */
