/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS Thread API
 */
#ifndef _NODEOS_THREAD_H_
#define _NODEOS_THREAD_H_

#include <nodeos/impl.h>
#include <nodeos/flow.h>

typedef struct {
	int stuff;
} *an_thread_t;

/* XXX compat */
#ifdef NODEOS_SPEC_COMPAT
#define an_Thread an_thread_t
#define an_threadId an_thread_current
#define an_threadStart an_thread_start
#define an_threadExit an_thread_exit
#define an_threadYield an_thread_yield
#define an_threadSleep an_thread_sleep
#define an_threadSetData an_thread_setdata
#define an_threadGetData an_thread_getdata
#define an_threadInterrupt an_thread_intr
#define an_threadBlockInterrupts an_thread_blockintr
#define an_threadUnblockInterrupts an_thread_unblockintr
#define an_threadSetPrio an_thread_setprio
#define an_threadGetPrio an_thread_getprio
#endif

/*
 * Return the identity of the currently running thread.
 */
an_thread_t an_thread_current(void);

/*
 * Start a thread from the given flow's thread pool.
 * Thread starts in the indicated function with the indicated argument.
 * Returns zero if thread was launched, an error otherwise.
 * XXX return value should be reconciled with other create functions,
 * I favor this style (return value an error, explicit out param for object).
 */
an_error_t an_thread_start(an_thread_t *out_thread, an_flow_t fid,
			   void (*f)(void *arg), void *arg);

/*
 * Exit the current thread.
 * Doesn't return (duh!)
 */
void an_thread_exit(void);

/*
 * Yield the processor to any other runnable thread.
 */
void an_thread_yield(void);

/*
 * Sleep for the indicated amount of time.
 * Currently blocks uninterruptibly.  This is bad.
 *
 * XXX it isn't clear what we should do after a thread interrupt handler
 * has been run.  Should the thread return to the sleep for the duration
 * of the indicated delay?  Should thread sleep return an EINTR sort of
 * error requiring the caller to recompute the delay and try again?
 * The latter case could be simplified by using an absolute time for the
 * sleep ala pthread_cond_timedwait.  This way the thread could loop on
 * the sleep until it returns EINVAL indicating that the sleep time has
 * expired.
 */
void an_thread_sleep(an_timespec_t *delay);

/*
 * Get/set thread specific data for the current thread.
 * Only one item per thread.  Subsequent setdata calls overwrite the
 * previous value.
 */
void *an_thread_getdata(void);
void an_thread_setdata(void *data);

/*
 * Make a thread run an interrupt handler.
 * If the specified thread is running, sleeping or blocked on a condition
 * variable and the thread allows interrupts, the thread is redirected to
 * run an interrupt handler.  How the interrupt handler is setup, what
 * its arguments are, and what the thread can do in the handler context
 * are all implementation specific.
 */
void an_thread_intr(an_thread_t thread);

/*
 * Block/unblock thread interrupts for the current thread.
 */
void an_thread_blockintr(void);
void an_thread_unblockintr(void);

/*
 * Get/set CPU priority for an arbitrary thread.
 * XXX not implemented.
 */
int an_thread_getprio(an_thread_t t);
void an_thread_setprio(an_thread_t t, int priority);

#endif /* _NODEOS_THREAD_H_ */
