/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * The Channel API
 */
#include <errno.h>
#include <nodeos/channel.h>

#include "nodeos.h"
#include "channel.h"
#include "demux_tlist.h"
#include "packet.h"
#include "mem.h"

an_chan_t
an_chan_create(an_chanattr_t *attrp, an_demuxkey_t key, an_chanrecv_t func)
{
	channel_t *chan;
	channel_attr_t *attr;
	demux_tlist_t *demux;
	int err;

	NODEOS_ENTER;

	/*
	 * Copy in the attribute structures and optional demux data
	 */
	attr = mem_alloc(sizeof *attr);
	if (attr) {
		NODEOS_COPYIN(attr, attrp, sizeof *attrp, &err);
	} else
		err = ENOMEM;

	/*
	 * Create the channel or cleanup on errors
	 */
	if (err == 0) {
		demux = (demux_tlist_t *)key;
		err = channel_create(attr, demux, func, &chan);
	} else {
		if (attr)
			mem_free(attr, sizeof *attr);
	}

	NODEOS_LEAVE;
	return err ? 0 : (an_chan_t)chan;
}

void
an_chan_destroy(an_chan_t chan)
{
	NODEOS_ENTER;
	channel_destroy((channel_t *)chan);
	NODEOS_LEAVE;
}


an_error_t
an_chan_send(an_chan_t chan, an_packet_t packet)
{
	int err;

	NODEOS_ENTER;
	err = channel_send((channel_t *)chan, (packet_t *)packet);
	NODEOS_LEAVE;
	return (an_error_t)err;
}

an_error_t
an_chan_getaddr(an_chan_t chan, an_chanaddr_t *addrp)
{
	channel_addr_t addr;
	int err;

	NODEOS_ENTER;
	err = channel_getaddr((channel_t *)chan, &addr);
	if (err == 0)
		NODEOS_COPYOUT(addrp, &addr, sizeof addr, &err);
	NODEOS_LEAVE;
	return (an_error_t)err;
}

an_error_t
an_chan_addmem(an_chan_t chan, int npages)
{
	int err;

	NODEOS_ENTER;
	err = channel_createbufs((channel_t *)chan, npages);
	NODEOS_LEAVE;
	return (an_error_t)err;
}

