/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * The Mempool API
 */
#include <nodeos/impl.h>
#include <nodeos/mempool.h>

#include "nodeos.h"
#include "mempool.h"

an_mempool_t
an_mempool_create(an_mempool_t parent, an_mempoolfull_func_t func)
{
	mempool_t *newpool;
	int err;

	NODEOS_ENTER;

	err = mempool_create((mempool_t *)parent, func, &newpool);

	NODEOS_LEAVE;
	return err ? 0 : (an_mempool_t)newpool;
}

an_error_t
an_mempool_destroy(an_mempool_t mpool)
{
	int err = 0;

	NODEOS_ENTER;
	err = mempool_destroy((mempool_t *)mpool);
	NODEOS_LEAVE;
	return (an_error_t)err;
}

an_mempool_t
an_mempool_current(void)
{
	an_mempool_t id;

	NODEOS_ENTER;
	id = (an_mempool_t)mempool_current();
	NODEOS_LEAVE;
	return id;
}

void *
an_mempool_alloc(an_mempool_t pool, size_t size)
{
	void *ptr;

	NODEOS_ENTER;
	ptr = mempool_alloc((mempool_t *)pool, size);
	NODEOS_LEAVE;
	return ptr;
}

void
an_mempool_free(an_mempool_t pool, void *ptr, size_t size)
{
	NODEOS_ENTER;
	mempool_free((mempool_t *)pool, ptr, size);
	NODEOS_LEAVE;
}
