/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * The Thread API
 */
#include <nodeos/thread.h>

#include "nodeos.h"
#include "flow.h"
#include "thread.h"

an_thread_t
an_thread_current(void)
{
	an_thread_t thread;

	NODEOS_ENTER;
	thread = (an_thread_t)thread_current();
	NODEOS_LEAVE;
	return thread;
}

an_error_t
an_thread_start(an_thread_t *out_thread, an_flow_t fid,
	       void (*f)(void *arg), void *arg)
{
	flow_t *flow = (flow_t *)fid;
	an_error_t err;

	NODEOS_ENTER;
	err = (an_error_t)thread_start(flow->tpool, 0, f, arg,
				     (thread_t **)out_thread);
	NODEOS_LEAVE;
	return err;
}

void
an_thread_exit(void)
{
	NODEOS_ENTER;
	thread_exit();
	NODEOS_LEAVE;
}

void
an_thread_yield(void)
{
	NODEOS_ENTER;
	thread_yield();
	NODEOS_LEAVE;
}

void
an_thread_sleep(an_timespec_t *delay)
{
	NODEOS_ENTER;
	thread_sleep(delay);
	NODEOS_LEAVE;
}

void
an_thread_setdata(void *data)
{
	NODEOS_ENTER;
	thread_setdata(data);
	NODEOS_LEAVE;
}

void *
an_thread_getdata(void)
{
	void *data;

	NODEOS_ENTER;
	data = thread_getdata();
	NODEOS_LEAVE;

	return data;
}

void
an_thread_intr(an_thread_t thread)
{
	NODEOS_ENTER;
	thread_interrupt((thread_t *)thread);
	NODEOS_LEAVE;
}

void
an_thread_blockintr(void)
{
	NODEOS_ENTER;
	thread_block_interrupt();
	NODEOS_LEAVE;
}

void
an_thread_unblockintr(void)
{
	NODEOS_ENTER;
	thread_unblock_interrupt();
	NODEOS_LEAVE;
}

void
an_thread_setprio(an_thread_t t, int priority)
{
}

int
an_thread_getprio(an_thread_t t)
{
	return 0;
}

/*
 * Implementation specific API calls
 */

void
an_thread_sethandler(an_thread_intr_t handler)
{
	NODEOS_ENTER;
	thread_sethandler(handler);
	NODEOS_LEAVE;
}
