/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _CHANNEL_IMPL_H_
#define _CHANNEL_IMPL_H_

#include <nodeos/channel.h>
#include "flow.h"
#include "threadpool.h"
#include "packet.h"
#include "ilock.h"

/* XXX */
#define channel_addr_t an_chanaddr_t
#define channel_attr_t an_chanattr_t
#define channel_deliver_t an_chanrecv_t

struct chanops;
struct chanbuf;
struct demux_tlist;

struct channel {
	struct channel	 *next;	   /* link of all channels for a flow */
	flow_t		 *flow;	   /* flow we are associated with */
	threadpool_t	 *tpool;   /* thread pool for our CPU resources */
	int		 curthreads;/* threads processing packets */
	channel_attr_t	 attr;	   /* channel attributes */
	struct chanops	 *ops;	   /* operations */
	struct chanbuf	 *rbufs;   /* receive buffers for inchans */
	void		 *data;	   /* channel-type specific data */
	struct demux_tlist *demux; /* demux information */
	channel_deliver_t  func;   /* delivery function */
	int		 flags;
	ilock_t		 lock;
};
typedef struct channel channel_t;

#define CHAN_DYING	1

#define channel_lock(c)		ilock_lock(&(c)->lock)
#define channel_unlock(c)	ilock_unlock(&(c)->lock)

#define channel_maxthreads(c)	((c)->attr.cpu.max)
#define channel_maxmemory(c)	((c)->attr.mem.npages*NODEOS_PAGESIZE)
#define channel_name(c)		((c)->attr.name)
#define channel_type(c)		((c)->attr.type)
#define channel_addrlen(c)	((c)->attr.addr.len)
#define channel_addrdata(c)	((c)->attr.addr.data)

struct chanops {
	char *name;
	void (*init)(void);
	int (*create)(channel_t *);
	void (*destroy)(channel_t *);
	void (*send)(channel_t *, packet_t *);
};

/*
 * In-channel receive buffers.
 */
struct chanbuf {
	char data[NODEOS_CHAN_BUFSIZE];
};

struct demux;
struct packet;

void channel_init(void);
channel_t *channel_current(void);
int channel_create(channel_attr_t *attr, struct demux_tlist *demux,
		   channel_deliver_t func, channel_t **out_channel);
void channel_destroy(channel_t *channel);
int channel_send(channel_t *channel, struct packet *packet);
int channel_getaddr(channel_t *channel, channel_addr_t *addr);
void channel_thread_start(channel_t *chan, struct packet *packet);
void channel_thread_exit(channel_t *chan);
int channel_packet_create(channel_t *chan, void *data, int dlen, int mustcopy,
			  packet_t **out_packet, struct chanbuf **out_buf);
int channel_createbufs(channel_t *chan, size_t size);
void channel_addbuf(channel_t *chan, struct chanbuf *cbuf);
void channel_destroybufs(channel_t *chan);
int channel_demux_add(channel_t *chan, struct demux **listp);
void channel_demux_remove(channel_t *chan, struct demux **listp);

#endif /* _CHANNEL_IMPL_H_ */
