/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _DEMUX_TLIST_IMPL_H_
#define _DEMUX_TLIST_IMPL_H_

/*
 * A list of <off, len, key> triples.  A packet matches if each of the
 * triples match.  A triple matches if "len" bytes of the packet data
 * at offset "off" are equal to "key".
 */
struct demux_triple;

struct demux_tlist {
	int		    ntriples;
	struct demux_triple *list[0];
};
typedef struct demux_tlist demux_tlist_t;

struct demux_triple {
	int	off;
	int	len;
	char	key[0];
};

int demux_tlist_create(int ntriples, demux_tlist_t **out_list);
void demux_tlist_destroy(demux_tlist_t *list);
int demux_tlist_prepend(demux_tlist_t *list, int off, int len, void *key);
int demux_tlist_append(demux_tlist_t *list, int off, int len, void *key);
int demux_tlist_copy(demux_tlist_t *list, int ntriples,
		     demux_tlist_t **out_list);
void demux_tlist_adjust(demux_tlist_t *list, int adjust);

int demux_tlist_match(void *ddata, void *pdata, int plen);

#endif /* _DEMUX_TLIST_IMPL_H_ */
