/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * EE to test channels
 */
#include "options.h"

#ifdef EE_CHANTEST

#include <stdio.h>
#include <string.h>
#include <assert.h>
#ifdef OSKIT
#include <oskit/net/ether.h>
#endif

#include <nodeos/impl.h>
#include <nodeos/flow.h>
#include <nodeos/thread.h>
#include <nodeos/channel.h>
#include <nodeos/misc.h>

static void ee_init(void *arg);
an_flowinit_t InitEE = { ee_init };

static void ee_packet1(an_packet_t packet);
static void ee_packet100(an_packet_t packet);
static void ee_realpacket(an_packet_t packet);
static an_chan_t chan1, chan100, eth;

int
ee_main(int argc, char **argv)
{
	an_chanattr_t attr;
#ifdef OSKIT
	struct enetaddr {
		unsigned char addr[6];
		unsigned short type;
	} bcastaddr = { {0xff, 0xff, 0xff, 0xff, 0xff, 0xff}, ETHERTYPE_ARP };
#endif

	printf("%p/%p: Channel test EE started\n",
	       an_flow_current(), an_thread_current());

	memset(&attr, 0, sizeof attr);

#ifdef OSKIT
	attr.name = "eth/eth0";
	attr.type = inChan;
	memcpy(attr.addr.data, &bcastaddr, sizeof bcastaddr);
	attr.addr.len = sizeof bcastaddr;
	attr.cpu.max = 2;
	attr.mem.npages = 1;
	eth = an_chan_create(&attr, 0, ee_realpacket);
	assert(eth != 0);
#endif

	attr.name = "clock";
	attr.type = inChan;
	*(int *)attr.addr.data = 100; /* XXX addr means delay */
	attr.addr.len = sizeof(int);
	attr.cpu.max = 1;
	attr.mem.npages = 1;
	chan100 = an_chan_create(&attr, 0, ee_packet100);

	*(int *)attr.addr.data = 1; /* XXX addr means delay */
	attr.addr.len = sizeof(int);
	attr.cpu.max = 1;
	attr.mem.npages = 1;
	chan1 = an_chan_create(&attr, 0, ee_packet1);

	return 0;
}

static int inpackets;

static void
ee_packet1(an_packet_t packet)
{
	inpackets++;
	an_packet_return(packet);
}

static void
ee_packet100(an_packet_t packet)
{
	static int count;
	int size = an_packet_size(packet);
	unsigned int *ts = an_packet_data(packet);
	an_timespec_t tspec;
	
	assert(size == sizeof *ts);

	an_gettime(&tspec);
	printf("%p/%p: packet %p@%u: got %d packets at %d seconds\n",
	       an_flow_current(), an_thread_current(), packet, *ts, inpackets,
	       tspec.tv_sec);
	an_packet_return(packet);
	if (++count == 20) {
#ifdef OSKIT
		an_chan_destroy(eth);
#endif
		printf("killing timer channel %p\n", chan1);
		if (chan1)
			an_chan_destroy(chan1);
/* if we kill our channel, the flow won't terminate */
#if 0
		printf("killing our channel %p\n", chan100);
		if (chan100)
			an_chan_destroy(chan100);
#else
		an_flow_destroy(an_flow_current());
#endif
		printf("WE ARE DEAD!\n");
	}
}

#ifdef OSKIT
static void
ee_realpacket(an_packet_t packet)
{
	int size = an_packet_size(packet);
	struct ether_header *eth = an_packet_data(packet);

	assert(size >= sizeof *eth);

	eth = an_packet_data(packet);
	printf("%p/%p: s=%02x:%02x:%02x:%02x:%02x:%02x, "
	       "d=%02x:%02x:%02x:%02x:%02x:%02x, t=%d\n",
	       an_flow_current(), an_thread_current(),
	       eth->ether_shost[0],eth->ether_shost[1],eth->ether_shost[2],
	       eth->ether_shost[3],eth->ether_shost[4],eth->ether_shost[5],
	       eth->ether_dhost[0],eth->ether_dhost[1],eth->ether_dhost[2],
	       eth->ether_dhost[3],eth->ether_dhost[4],eth->ether_dhost[5],
	       eth->ether_type);
	an_packet_return(packet);
}
#endif

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	char *argv[] = { "ChantestEE" }; 

	status = ee_main(argc, argv);
}

#endif
