/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Reply to UDP ping packets (shamelessly stolen from OSKit pingreply).
 */
#include "options.h"

#ifdef EE_PINGREPLY

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/if_ether.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>

#include <nodeos/impl.h>
#include <nodeos/flow.h>
#include <nodeos/thread.h>
#include <nodeos/channel.h>

#define RUNFOR	30	/* run for this many seconds */
static unsigned char myip[4] = { 155, 99, 214, 158 };


/*
 * For some reason the headers define ETHER_MAX_LEN as
 * (ETH_FRAME_LEN + ETH_CRC_LEN), but then don't define ETH_CRC_LEN.
 */
#ifdef __linux__
#ifndef ETH_CRC_LEN
#define ETH_CRC_LEN ETHER_CRC_LEN
#endif
#endif

/*
 * XXX hack arp tab cuz I don't want to do the protocol
 */
struct arpentry {
	unsigned char ethaddr[6];
	unsigned char ipaddr[4];
	an_chan_t chan;
} arptab[] = {
	/* moab */
	{ { 0x00, 0x60, 0xf5, 0x04, 0x00, 0x53 }, { 155,  99, 214,  88 }, 0 },
	/* risky */
	{ { 0x00, 0x00, 0xc0, 0x26, 0x0c, 0x9f }, { 155,  99, 214, 162 }, 0 },
	/* quirky */
	{ { 0x00, 0x00, 0xc0, 0x47, 0x0c, 0x9f }, { 155,  99, 214, 164 }, 0 }
};
static int arptabsize = sizeof arptab / sizeof arptab[0];

static void ee_init(void *arg);
an_flowinit_t InitEE = { ee_init };

static char *iptoa(unsigned long x);
static void ee_die(an_packet_t packet);
static void ee_ip(an_packet_t packet);
static void ee_arp(an_packet_t packet);
static void setup_channels(void);

static an_demuxkey_t arpdemux, ipdemux;
static an_chan_t ipinchan, arpinchan, arpoutchan, die;
static unsigned int myipaddr;
static int gotpackets, sentpackets, requests, replies, dying;

struct enetaddr {
	unsigned char addr[6];
	unsigned short type;
};

struct enetaddr bcastaddr = {
	{0xff, 0xff, 0xff, 0xff, 0xff, 0xff}, ETHERTYPE_ARP
};
struct enetaddr myaddr = {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, ETHERTYPE_IP
};
struct enetaddr arpoutaddr = {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, ETHERTYPE_ARP
};
struct enetaddr ipoutaddr = {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, ETHERTYPE_IP
};

int
ee_main(int argc, char **argv)
{
	assert(argc >= 1);

	printf("%p/%p: %s started\n",
	       an_flow_current(), an_thread_current(), argv[0]);
	myipaddr = *(unsigned int *)myip;

	/*
	 * Setup channels
	 */
	setup_channels();

	printf("%s (%02x:%02x:%02x:%02x:%02x:%02x) ready\n",
	       iptoa(*(unsigned int *)myip),
	       myaddr.addr[0], myaddr.addr[1], myaddr.addr[2],
	       myaddr.addr[3], myaddr.addr[4], myaddr.addr[5]);

#ifdef GPROF
	oskit_start_gprof();
#endif
#ifdef OSKIT
	{
		extern int memlogging;
		memlogging = 1;
	}
#endif        
	return 0;
}

static void
swapn(unsigned char a[], unsigned char b[], int n)
{
	int i;
	unsigned char t[n];

	for (i = 0; i < n; i++) {
		t[i] = a[i];
		a[i] = b[i];
		b[i] = t[i];
	}
}


/* The address we are passed is in network order. */
static char *
iptoa(unsigned long x)
{
	static char buf[4*4];		/* nnn.nnn.nnn.nnnz */
	unsigned char *p = (unsigned char *)&x;

	sprintf(buf, "%d.%d.%d.%d", p[0], p[1], p[2], p[3]);
	return buf;
}

static unsigned short
ipcksum(void *buf, int nbytes)
{
	unsigned long sum;

	sum = 0;
	while (nbytes > 1) {
		sum += *(unsigned short *)buf;
		buf += 2;
		nbytes -= 2;
	}
	/* Add in odd byte if any. */
	if (nbytes == 1)
		sum += *(char *)buf;

	sum = (sum >> 16) + (sum & 0xffff);     /* hi + lo */
	sum += (sum >> 16);                     /* add carry, if any */

	return (~sum);
}

static an_chan_t
find_channel(struct ip *ip)
{
	static struct arpentry *lastent = arptab;
	struct arpentry *aent;

	/*
	 * Find the table entry
	 */
	aent = lastent;
	do {
		if (memcmp(aent->ipaddr, &ip->ip_src.s_addr, 4) == 0) {
			lastent = aent;
			return aent->chan;
		}
		if (++aent == &arptab[arptabsize])
			aent = arptab;
	} while (aent != lastent);

	return 0;
}

static void
setup_channels(void)
{
	struct arpentry *aent;
	an_chanattr_t attr;
	an_error_t err;
	struct ether_arp *arp = 0;
	struct ip *ip = 0;
	unsigned int val_l;
	unsigned short val_s;
	unsigned char val_c;

	memset(&attr, 0, sizeof attr);

	/*
	 * Setup a clock channel to time us out in RUNFOR seconds
	 */
	attr.name = "clock";
	attr.type = inChan;
	*(int *)attr.addr.data = (RUNFOR * 100); /* XXX addr means delay */
	attr.addr.len = sizeof(int);
	attr.cpu.max = 1;
	attr.mem.npages = 1;
	die = an_chan_create(&attr, 0, ee_die);
	assert(die != 0);
	printf("%p: clockchannel for %d seconds\n", die, RUNFOR);
	
	/*
	 * Setup in channel for broadcast (arp) packets
	 * First, create a demux to match ARP requests.
	 */
	arpdemux = an_demuxkey_create(an_flow_current());
	assert(arpdemux != 0);
	val_s = htons(ARPHRD_ETHER);
	err = an_demuxkey_match(arpdemux, (int)&arp->arp_hrd, sizeof val_s,
			       &val_s, 0);
	assert(err == 0);
	val_s = htons(ETHERTYPE_IP);
	err = an_demuxkey_match(arpdemux, (int)&arp->arp_pro, sizeof val_s,
			       &val_s, 0);
	assert(err == 0);
	val_s = htons(ARPOP_REQUEST);
	err = an_demuxkey_match(arpdemux, (int)&arp->arp_op, sizeof val_s,
			       &val_s, 0);
	assert(err == 0);
	val_l = myipaddr;
	err = an_demuxkey_match(arpdemux, (int)&arp->arp_tpa, sizeof val_l,
			       &val_l, 0);
	assert(err == 0);

	attr.name = "eth/eth0";
	attr.type = inChan;
	bcastaddr.type = htons(bcastaddr.type);
	memcpy(attr.addr.data, &bcastaddr, sizeof bcastaddr);
	attr.addr.len = sizeof bcastaddr;
	attr.cpu.max = 2;
	attr.mem.npages = 1;
	arpinchan = an_chan_create(&attr, arpdemux, ee_arp);
	assert(arpinchan != 0);
	printf("%p: inchannel for %02x:%02x:%02x:%02x:%02x:%02x, type=%x\n",
	       arpinchan,
	       bcastaddr.addr[0], bcastaddr.addr[1], bcastaddr.addr[2],
	       bcastaddr.addr[3], bcastaddr.addr[4], bcastaddr.addr[5],
	       ntohs(bcastaddr.type));

	/*
	 * Setup in channel for ICMP/IP packets
	 * Again, we first build up the demux.  Note that due to the
	 * potentially variable-length IP header, we cannot check the ICMP
	 * type.  So all we can demux on is the IP type and even that might
	 * not be reliable since we cannot verify the checksum.
	 */
	ipdemux = an_demuxkey_create(an_flow_current());
	assert(ipdemux != 0);
	val_c = IPPROTO_ICMP;
	err = an_demuxkey_match(ipdemux, (int)&ip->ip_p, sizeof val_c,
			       &val_c, 0);
	assert(err == 0);

	attr.name = "eth/eth0";
	attr.type = inChan;
	myaddr.type = htons(myaddr.type);
	memcpy(attr.addr.data, &myaddr, sizeof myaddr);
	attr.addr.len = sizeof myaddr;
	attr.cpu.max = 8;
	attr.mem.npages = 8;
	ipinchan = an_chan_create(&attr, ipdemux, ee_ip);
	assert(ipinchan != 0);

	/*
	 * Get our interface address before reporting on channel
	 */
	err = an_chan_getaddr(ipinchan, &attr.addr);
	assert(err == 0);
	memcpy(&myaddr, attr.addr.data, sizeof myaddr);

	printf("%p: inchannel for %02x:%02x:%02x:%02x:%02x:%02x, type=%x\n",
	       arpinchan,
	       myaddr.addr[0], myaddr.addr[1], myaddr.addr[2],
	       myaddr.addr[3], myaddr.addr[4], myaddr.addr[5],
	       ntohs(myaddr.type));

	/*
	 * Setup possible output channels
	 */
	ipoutaddr.type = htons(ipoutaddr.type);
	for (aent = arptab; aent < &arptab[arptabsize]; aent++) {
		memset(&attr, 0, sizeof attr);
		attr.name = "eth/eth0";
		attr.type = outChan;
		memcpy(ipoutaddr.addr, aent->ethaddr, sizeof ipoutaddr.addr);
		memcpy(attr.addr.data, &ipoutaddr, sizeof ipoutaddr);
		attr.addr.len = sizeof ipoutaddr;
		aent->chan = an_chan_create(&attr, 0, 0);
		assert(aent->chan != 0);

		printf("%p: outchannel to %s (%02x:%02x:%02x:%02x:%02x:%02x), type=%x\n",
		       aent->chan, iptoa(*(unsigned int *)aent->ipaddr),
		       aent->ethaddr[0], aent->ethaddr[1], aent->ethaddr[2],
		       aent->ethaddr[3], aent->ethaddr[4], aent->ethaddr[5],
		       ntohs(ipoutaddr.type));
	}
}

static void
ee_arp(an_packet_t packet)
{
	int psize = an_packet_size(packet);
	void *data = an_packet_data(packet);
	struct ether_arp *arp;
	struct in_addr ip;
	an_error_t err;

	if (dying)
		goto done;

	gotpackets++;

	if (psize < sizeof *arp) {
#ifdef VERBOSE
		printf("yow! ARP packet too small (%d < %d)\n",
		       psize, sizeof *arp);
#endif
		goto done;
	}


	arp = (struct ether_arp *)data;
	ip.s_addr = *(unsigned long *)arp->arp_tpa;
	assert(ntohs(arp->arp_hrd) == ARPHRD_ETHER);
	assert(ntohs(arp->arp_pro) == ETHERTYPE_IP);
	assert(ntohs(arp->arp_op) == ARPOP_REQUEST);
	assert(ip.s_addr == myipaddr);

	/* Send the reply. */
	arp->arp_op = htons(ARPOP_REPLY);
	swapn(arp->arp_spa, arp->arp_tpa, sizeof arp->arp_tpa);
	swapn(arp->arp_sha, arp->arp_tha, sizeof arp->arp_tha);
	memcpy(arp->arp_sha, myaddr.addr, sizeof myaddr.addr);

#ifdef VERBOSE
	printf("sending ARP reply to %s (%02x:%02x:%02x:%02x:%02x:%02x)\n",
	       iptoa(*(unsigned int *)arp->arp_tpa),
	       arp->arp_tha[0], arp->arp_tha[1], arp->arp_tha[2],
	       arp->arp_tha[3], arp->arp_tha[4], arp->arp_tha[5]);
#endif

	/*
	 * Got a legit packet to send.
	 */
	if (memcmp(arpoutaddr.addr, arp->arp_tha, sizeof arpoutaddr.addr) != 0) {
		an_chanattr_t attr;

		if (arpoutchan)
			an_chan_destroy(arpoutchan);
		else
			arpoutaddr.type = htons(arpoutaddr.type);
		memset(&attr, 0, sizeof attr);
		attr.name = "eth/eth0";
		attr.type = outChan;
		memcpy(arpoutaddr.addr, arp->arp_tha, sizeof arpoutaddr.addr);
		memcpy(attr.addr.data, &arpoutaddr, sizeof arpoutaddr);
		attr.addr.len = sizeof arpoutaddr;
		arpoutchan = an_chan_create(&attr, 0, 0);
		assert(arpoutchan != 0);
	}
	err = an_chan_send(arpoutchan, packet);
	assert(err == 0);
	sentpackets++;

done:
	an_packet_return(packet);
}

static void
ee_ip(an_packet_t packet)
{
	int psize = an_packet_size(packet);
	void *data = an_packet_data(packet);
	int ipdata[ETHER_MAX_LEN/sizeof(int)+1];
	static int ipid = 0xb;		/* something random */
	struct ip *ip;
	int hlen;
	struct icmp *icmp;
	an_chan_t outchan;
	an_error_t err;

	if (dying)
		goto done;

	gotpackets++;

	if (psize < sizeof *ip) {
#ifdef VERBOSE
		printf("yow! IP packet too small (%d < %d)\n",
		       psize, sizeof *ip);
#endif
		goto done;
	}

	/* XXX deal with fragments. */

	ip = (struct ip *)data;
	hlen = ip->ip_hl << 2;

	/*
	 * Copy out from ip header on to ensure proper alignment
	 * on machines that are not byte addressable!
	 */
	memcpy(ipdata, data, ntohs(ip->ip_len));
	ip = (struct ip *) ipdata;

	assert(ip->ip_p == IPPROTO_ICMP);
	if (ipcksum(ip, hlen) != 0)
		goto done;		/* bad cksum */

	icmp = (struct icmp *)((char *)ip + hlen);
	if (icmp->icmp_type != ICMP_ECHO
	    || ipcksum(icmp, ntohs(ip->ip_len) - hlen) != 0)
		goto done;		/* bad cksum or not ICMP_ECHO */

	/*
	 * Find/create a channel to send on
	 */
	outchan = find_channel(ip);
	if (outchan == 0) {
#ifdef VERBOSE
		printf("yow! I do not know about %s\n",
		       iptoa(ip->ip_src.s_addr));
#endif
		goto done;
	}

	requests++;

	/* Send the reply. */
	icmp->icmp_type = ICMP_ECHOREPLY;
	/* Another optimization.  Only the icmp type field in the reply
	 * changes, so we can use this apriori knowledge to determine
	 * the new checksum based upon the old checksum.  Eliminates
	 * touching all of the data in the packet once more */
#if 0
	icmp->icmp_cksum = 0;
	icmp->icmp_cksum = ipcksum(icmp, ntohs(ip->ip_len) - hlen);
#else
	if (icmp->icmp_cksum >= 65527)
		icmp->icmp_cksum += (unsigned short) 9;
	else
		icmp->icmp_cksum += (unsigned short) 8;
#endif

	ip->ip_id = htons(ipid++);

	/*
	 * We can't just swap the addresses,
	 * since broadcast packets didn't come to our address.
	 */
	ip->ip_dst = ip->ip_src;
	ip->ip_src.s_addr = myipaddr;

	ip->ip_sum = 0;
	ip->ip_sum = ipcksum(ip, hlen);

	/*
	 * Copy data back into original data buffer.
	 */
	memcpy(data, ip, ntohs(ip->ip_len));

	replies++;

#ifdef VERBOSE
	printf("sending ICMP reply to %s on %p\n",
	       iptoa(ip->ip_dst.s_addr), outchan);
#endif

	err = an_chan_send(outchan, packet);
	assert(err == 0);
	sentpackets++;

done:
	an_packet_return(packet);
}

static void
ee_die(an_packet_t packet)
{
	dying = 1;
#ifdef GPROF
	oskit_stop_gprof();
#endif
	printf("All done, %d/%d packets received/sent, %d/%d request/reply\n",
	       gotpackets, sentpackets, requests, replies);

	an_packet_return(packet);
	if (ipinchan)
		an_chan_destroy(ipinchan);
	if (arpoutchan)
		an_chan_destroy(arpoutchan);
	if (arpinchan)
		an_chan_destroy(arpinchan);
	an_flow_destroy(an_flow_current());
}

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	char *argv[] = { "PingreplyEE" }; 

	status = ee_main(argc, argv);
}

#endif
