/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _FLOW_IMPL_H_
#define _FLOW_IMPL_H_

#include <pthread.h>
#include <nodeos/impl.h>

#include "hnode.h"
#include "resource.h"
#include "synch.h"

struct credential;
struct threadpool;
struct channel;

/*
 * The flow: rendezvous point for threads and resources.
 *
 * Every flow is part of the flow hierarchy as well as possibly associated
 * with a mempool.
 */

/* XXX */
#define flow_init_t	an_flowinit_t
#define flow_term_t	an_flowterm_func_t

struct flow {
	struct hnode		me;	/* flow hier link (must be first) */
	struct credential	*cred;		/* credentials */
	struct mempool		*mpool;		/* mempool */
	struct threadpool	*tpool;		/* thread pool */
	struct channel		*chanlist;	/* list of channels */
	int			flags;		/* various flags */
	flow_term_t		termfunc;	/* termination function */
	resource_t		res;		/* reserved resources */
	mutex_t			lock;		/* mutual exclusion */
};
typedef struct flow flow_t;

#define FLOW_INUPCALL	1
#define FLOW_DYING	2

#define flow_lock(f)	mutex_lock(&(f)->lock)
#define flow_unlock(f)	mutex_unlock(&(f)->lock)

#define flow_parent(f)	((flow_t *)hnode_parent(&(f)->me))
#define flow_child(f)	((flow_t *)hnode_child(&(f)->me))
#define flow_sibling(f)	((flow_t *)hnode_sibling(&(f)->me))

void flow_init(flow_init_t ifunc, flow_term_t tfunc);

/*
 * Return reference to current flow context.
 */
flow_t *flow_current(void);

/*
 * Create a new flow.
 * The new flow 
 * the indicated parent flow, memory pool and thread scheduler.
 */
int flow_create(struct credential *cred,
		cpuspec_t *cspec, memspec_t *mspec,
		flow_init_t initfunc, flow_term_t termfunc,
		struct flow **out_flow);

/*
 * Destroy the indicated flow.
 * Removes its memory resources from the associated memory pool.
 */
int flow_destroy(flow_t *flow);

void flow_addchannel(flow_t *flow, struct channel *chan);
void flow_removechannel(flow_t *flow, struct channel *chan);

#endif /* _FLOW_IMPL_H_ */

