#
# Copyright (c) 1999 University of Utah and the Flux Group.
# All rights reserved.
# This file is part of Janos.
# 
# Janos is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
# 
# Janos is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, in the file
# COPYING.  If that file is not present, write to the Free
# Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
# 02111-1307, USA
# 
# The University of Utah and the Flux Group requests users to
# return to flux-dist@cs.utah.edu any improvements that they make.
#

EESRCS =	ee_null.c ee_flowtest.c ee_chantest.c ee_pingreply.c \
		ee_socktest.c

SRCS =		main.c flow.c threadpool.c thread.c credential.c resource.c \
		mempool.c mem.c channel.c packet.c synch.c demux.c \
		demux_tlist.c hnode.c misc.c \
		chan_clock.c chan_eth.c chan_socket.c \
		api_credential.c api_flow.c api_thread.c api_mempool.c \
		api_channel.c api_demuxkey.c api_packet.c api_synch.c api_misc.c

ifeq ($(HOST_OS), oskit)
ARCHSRCS = 	oskit.c oskit_mem.c
ARCH_LIBS =	-loskit_startup -loskit_linux_dev -loskit_bootp \
		-loskit_freebsd_net # -loskit_memdebug
ARCH_PLIBS =	-loskit_startup_p -loskit_linux_dev_p -loskit_bootp_p \
		-loskit_freebsd_net_p -loskit_memfs_p -loskit_fsnamespace_p \
		-loskit_gprof # -loskit_memdebug_p
CFLAGS +=	-DOSKIT # -DNODEOS_SPEC_COMPAT
DEPINCS =	-I$(OSKIT_INSTDIR)/include -I$(OSKIT_INSTDIR)/include/oskit/c \
		-I$(OSKIT_INSTDIR)/include/oskit/threads
endif

ifeq ($(HOST_OS), linux-gnu)
ARCHSRCS =	linux.c unix_eth.c
endif

ifeq ($(HOST_OS), solaris2.7)
ARCHSRCS =	solaris.c unix_eth.c
ARCH_LIBS =	-lposix4 -lpthread -lsocket
endif

ifeq ($(HOST_OS), freebsd3.0)
ARCHSRCS =	freebsd.c unix_eth.c
endif

#
# hack to avoid conflicts between -lgcc and -lfreebsd_c(divsi3.o)
#
ifeq ($(HOST_ARCH), arm32)
ARCH_LIBS +=	-loskit_threads -loskit_dev
ARCH_PLIBS +=	-loskit_threads_p -loskit_dev_p
endif


FILES = 	$(SRCS) $(ARCHSRCS) $(EESRCS)
DEPSRCS =	$(addprefix $(SRCDIR)/,$(FILES))


VPATH := $(SRCDIR)

CFLAGS += -g -Wall $(ARCH_CFLAGS) -I$(NODEOS_SRCDIR)
CPPFLAGS = $(ARCH_CFLAGS) -I$(NODEOS_SRCDIR)

# C source files
OBJS := $(patsubst %.c,%.o,$(notdir $(FILES)))
POBJS := $(patsubst %.c,%.po,$(notdir $(FILES)))
OOBJS := $(patsubst %.c,%.oo,$(notdir $(FILES)))

OLD= $(LD)
PLD= $(LD)


# How to compile them.
%.o: %.c
	$(CC) $(CFLAGS) -c  $< -o $@ 

# How to build profiled object files.
%.po: %.c
	$(CC) $(CFLAGS) -pg -DGPROF -c $< -o $@

# How to build super optimized object files.
%.oo: %.c
	$(CC) $(CFLAGS) -O4 -DNDEBUG -c $< -o $@



all: $(TARGET)

$(TARGET): $(OBJS)
	$(LD) $(LDFLAGS) -o $@ $(OBJS) $(ARCH_LIBS)

$(TARGET)_o: $(OOBJS)
	$(OLD) $(LDFLAGS) -o $@ $(OOBJS) $(ARCH_LIBS)

$(TARGET)_p: $(POBJS)
	$(PLD) -pg $(LDFLAGS) -o $@ $(POBJS) $(ARCH_PLIBS)

Image_p: nodeos_p
	mkmbimage nodeos_p nodeos_p
	mv Image Image_p

nodeos_arm: nodeos
	cp nodeos nodeos_arm
	$(STRIP) --strip-debug nodeos_arm

Image_arm: nodeos_arm
	$(OSKIT_INSTDIR)/bin/mkofwimage -o Image_arm nodeos_arm

clean:
	rm -f .depend core *.core *.o *.po *.oo nodeos nodeos_p nodeos_o

distclean: clean
	rm -f GNUmakefile

.PHONY: depend
depend: 
	$(CC) $(CFLAGS) -E -M $(CPPFLAGS) $(DEPSRCS) $(DEPINCS) > .depend


-include .depend
