/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _SYNCH_IMPL_H_
#define _SYNCH_IMPL_H_

#include "ilock.h"
#include <pthread.h>

struct _mutex {
	pthread_mutex_t	mutex;
};
typedef struct _mutex mutex_t;

struct _cond {
	pthread_cond_t	cond;
	ilock_t		lock;
};
typedef struct _cond cond_t;

void mutex_init(mutex_t *mutex);
void mutex_deinit(mutex_t *mutex);
int mutex_create(mutex_t **out_mutex);
void mutex_lock(mutex_t *mutex);
int mutex_trylock(mutex_t *mutex);
void mutex_unlock(mutex_t *mutex);
int mutex_destroy(mutex_t *mutex);

#define cond_lock(c)	ilock_lock(&(c)->lock)
#define cond_unlock(c)	ilock_unlock(&(c)->lock)
#define cond_trylock(c)	ilock_trylock(&(c)->lock)

void cond_init(cond_t *cond);
void cond_deinit(cond_t *cond);
int cond_create(cond_t **out_cond);
void cond_wait(cond_t *cond, mutex_t *mutex);
void cond_signal(cond_t *cond);
void cond_broadcast(cond_t *cond);
int cond_destroy(cond_t *cond);

#endif /* _SYNCH_IMPL_H_ */
