/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _THREAD_IMPL_H_
#define _THREAD_IMPL_H_

#include <pthread.h>
#include <nodeos/impl.h>

#include "ilock.h"
#include "synch.h"
#include "resource.h"

/*
 * Stuff That Doesn't Belong
 * XXX hack around pthread deficiencies:
 *	OSKit doesn't support attribute scope stuff currently,
 *		the default (only!) scope is system.
 *	FreeBSD doesn't support cancelation,
 *		the EE had really better cooperate!
 */
#ifndef PTHREAD_CANCEL_ASYNCHRONOUS
#define PTHREAD_CANCEL_ASYNCHRONOUS	0
#define PTHREAD_CANCEL_DEFERRED		0
#define pthread_setcanceltype(t, sp)	(*(sp) = 0)
#define pthread_cancel(t)		0
#define pthread_testcancel()
#undef PTHREAD_SCOPE_SYSTEM /* XXX freebsd defines this but doesn't impl */
#endif

#ifndef PTHREAD_SCOPE_SYSTEM
#define pthread_attr_setscope(a,s) (0)
#endif

/* XXX */
#define thread_handler_t	an_thread_intr_t

struct threadpool;
struct channel;

struct thread {
	struct thread *next;	 /* threadpool link */
	struct threadpool *pool; /* our threadpool */
	struct channel *chan;	 /* channel we are processing for */
	pthread_t id;		 /* pthread ID */
	icond_t waiting;	 /* a place to wait for a thread */
	thread_handler_t handler;/* interrupt/trap handler */
	ilock_t lock;		 /* lock to protect interrupt state */
	cond_t *cond;		 /* non-zero if in cond_wait */
	int flags;		 /* various flags */
};
typedef struct thread thread_t;

#define THREAD_INKERNEL	0x01	/* thread is in the NodeOS */
#define THREAD_WAITING	0x02	/* someone is waiting for thread to die */
#define THREAD_INTSON	0x04	/* interrupts should be allowed */

#define thread_lock(t)		ilock_lock(&(t)->lock)
#define thread_unlock(t)	ilock_unlock(&(t)->lock)

void thread_init(void);
thread_t *thread_current(void);
int thread_checkattrs(cpuspec_t *cspec);
int thread_start(struct threadpool *tpool, struct channel *chan,
		 void (*func)(void *arg), void *arg, thread_t **out_thread);
int thread_run(struct threadpool *tpool, void (*func)(void *arg), void *arg,
	       an_timespec_t *timo);
int thread_start_kernel(struct threadpool *tpool, struct channel *chan,
			void (*func)(void *arg), void *arg,
			thread_t **out_thread);
void thread_kill(thread_t *thread);
void thread_exit(void);
void thread_yield(void);
void thread_sleep(an_timespec_t *delay);
void *thread_getdata(void);
void thread_setdata(void *);
void thread_interrupt(thread_t *thread);
void thread_block_interrupt(void);
void thread_unblock_interrupt(void);
void thread_sethandler(thread_handler_t handler);
void thread_condwait(cond_t *cond);

#endif /* _THREAD_IMPL_H_ */

