/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _THREADPOOL_H_
#define _THREADPOOL_H_

#include "ilock.h"
#include "resource.h"

struct flow;
struct chan;
struct thread;
struct channel;

/*
 * The threadpool: a group of threads sharing the same scheduling params
 */

struct threadpool {
	struct flow *flow;		/* associated flow */
	struct chan *chan;		/* associated channel */
	int curthreads;			/* current # of threads in pool */
	int flags;			/* various flags */
	struct thread *threads;		/* list of active threads */
	cpuspec_t params;		/* resource parameters */
	ilock_t lock;			/* lock for entire pool */
};
typedef struct threadpool threadpool_t;

#define TPOOL_DYING	1

#define threadpool_lock(t)	ilock_lock(&(t)->lock)
#define threadpool_unlock(t)	ilock_unlock(&(t)->lock)
#define threadpool_islocked(t)	ilock_islocked(&(t)->lock)

int threadpool_create(struct flow *flow, struct chan *chan,
		      cpuspec_t *cspec, threadpool_t **out_pool);
void threadpool_kill(threadpool_t *tpool, struct channel *chan);
void threadpool_destroy(threadpool_t *tpool);
void threadpool_cleanup(threadpool_t *tpool);
#endif /* _THREADPOOL_H_ */
